"""Run all tests auto discovered by unittest.TestLoader"""

import unittest
import sys


if __name__ == "__main__":
    all_tests = unittest.TestLoader().discover('test')
    verbosity = 1
    for argv in sys.argv:
        if "-v" in argv:
            verbosity = argv.count("v") + 1
    unittest.TextTestRunner(verbosity=verbosity).run(all_tests)



#!/usr/bin/env python

"""
withsqlite - uses an sqlite db as a back end for a dict-like object,
kind of like shelve but with json and sqlite3.

Copyright 2011-2013 James Vasile
Released under the GNU General Public License, version 3 or later.
See https://www.gnu.org/licenses/gpl-3.0.html for terms.

Repo is at <http://github.com/jvasile/withsqlite>.  Patches welcome!

This file was developed as part of planeteria
<http://github.com/jvasile/planeteria>

"""
from __future__ import print_function, unicode_literals
import os
import sqlite3
import time
from contextlib import contextmanager
try:
    import simplejson as json
except ImportError:
    import json


def to_json(python_object):
    if isinstance(python_object, time.struct_time):
        return {'__class__': 'time.asctime',
                '__value__': time.asctime(python_object)}

    return {'__class__': 'basestring',
            '__value__': str(python_object)}


class sqlite_db():
    """
    Backends a dict on an sqlite db.  This class aims to present like a
    dict wherever it can.

    USE:
    import sqlite_db from withsqlite
    with sqlite_db("filename") as db:
       db['aaa'] = {'test':'ok'}
       print db.items()

    Specify a table to have one sqlite db hold multiple dicts:

    with sqlite_db("filename", table="fruit") as db:
       db['citrus'] = ['orange', 'grapefruit']
       print db.items()

    If you change the dict in any way, its state will differ from the
    state of the sqlite database.  Changes are committed to disk when you
    close the database connection or (if you've set autocommit to True) after
    each assignment.
    """

    def __init__(self, fname, dir=None, autocommit=False, table="store", check_same_thread = True):
        if dir:
            fname = os.path.join(os.path.abspath(os.path.normpath(dir)), fname)
        self.fname = '{}.sqlite3'.format(fname)
        self.autocommit = autocommit
        self.table = table
        self.conn = None
        self.crsr = None
        self.check_same_thread = check_same_thread

    def _connect(self):
        if not self.conn:
            self.make_db()
            if self.autocommit:
                self.conn = sqlite3.connect(self.fname, isolation_level=None, check_same_thread = self.check_same_thread)
            else:
                self.conn = sqlite3.connect(self.fname, check_same_thread = self.check_same_thread)
        if not self.crsr:
            self.crsr = self.conn.cursor()

    def make_db(self):
        conn = sqlite3.connect(self.fname, check_same_thread = self.check_same_thread)
        crsr = conn.cursor()
        qry = "create table if not exists {} (key text unique, val text)"
        crsr.execute(qry.format(self.table))
        conn.commit()
        crsr.close()

    def isstring(self, value):
        "Check if the value is a string"
        try:
            return isinstance(value, basestring)
        except NameError:
            return isinstance(value, str)

    def jsonize(self, val):
        "Use json.dumps to jsonize all values."
        return json.dumps(val, default=to_json, sort_keys=True, indent=3)

    def has_key(self, key):
        print("W601 .has_key() is deprecated, use 'in'")
        return self.__contains__(key)

    def keys(self):
        """a.keys()   a copy of a's list of keys"""
        self.crsr.execute("select key from {}".format(self.table))
        return [f[0] for f in self.crsr.fetchall()]

    def values(self):
        """a.values()     a copy of a's list of values"""
        self.crsr.execute("select val from {}".format(self.table))
        return [json.loads(f[0]) for f in self.crsr.fetchall()]

    def items(self):
        """a.items()  a copy of a's list of (key, value) pairs"""
        self.crsr.execute("select * from {}".format(self.table))
        return [(f[0], json.loads(f[1])) for f in self.crsr.fetchall()]

    def get(self, k, x=None):
        """a.get(k[, x])  a[k] if k in a, else x """
        try:
            return self.__getitem__(k)
        except KeyError:
            return x

    def clear(self):
        """a.clear()  remove all items from a"""
        self.crsr.execute("delete from {}".format(self.table))

    def begin(self):
        "Starts the transaction"
        self._connect()

    def save(self):
        "Ends the transaction"
        self.conn.commit()
        self.crsr.close()

    @contextmanager
    def transaction(self):
        """You can use an instance of sqlite_db as follow:
        >>> a = sqlite_db("test")
            with a.transaction():
                a['key'] = 'value'
        """
        self.begin()
        yield
        self.save()

    def __enter__(self):
        self.begin()
        return self

    def __exit__(self, type, value, traceback):
        self.save()

    def __setitem__(self, key, val):
        """a[k] = v   set a[k] to v   """
        try:
            if val == self.__getitem__(key):
                return
            qry = "update or fail {} set val=? where key==?"
            self.crsr.execute(qry.format(self.table), [self.jsonize(val), key])
        except KeyError:
            qry = "insert into {} values (?, ?)"
            self.crsr.execute(qry.format(self.table), [key, self.jsonize(val)])

    def __getitem__(self, key):
        """a[k]   the item of a with key k    (1), (10)"""
        qry = 'select val from {} where key=?'
        self.crsr.execute(qry.format(self.table), [key])
        try:
            f = self.crsr.fetchone()[0]
        except TypeError:
            raise KeyError(key)
        return json.loads(f)

    def __contains__(self, key):
        """k in a     True if a has a key k, else False
        k not in a     Equivalent to not k in a"""
        qry = "select COUNT(*) from {} where key=?"
        self.crsr.execute(qry.format(self.table), [key])
        return self.crsr.fetchone()[0] != 0

    def __len__(self):
        """len(a)     the number of items in a"""
        self.crsr.execute("select COUNT(*) from {}".format(self.table))
        return self.crsr.fetchone()[0]

    def __delitem__(self, key):
        """del a[k]   remove a[k] from a"""
        qry = "delete from {} where key=?"
        self.crsr.execute(qry.format(self.table), [key])

    def __repr__(self):
        r = []
        for k, v in self.items():
            r.append('{!r}: {!r}'.format(k, v))
        return '{{{}}}'.format(", ".join(r))



#!/usr/bin/env python
from distutils.core import setup, Command


class TestRunner(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys
        import subprocess
        errno = subprocess.call([sys.executable, 'runtests.py'])
        raise SystemExit(errno)

setup(
    name='withsqlite',
    version='0.1',
    author='James Vasile',
    #author_email='TBC',
    url='http://github.com/jvasile/withsqlite',
    long_description=open('README.mdwn').read(),
    py_modules=['withsqlite'],
    cmdclass={'test': TestRunner},
    license="GPL-3",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GPL-3',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
)



from __future__ import print_function, unicode_literals
import unittest
import tempfile
from withsqlite import sqlite_db


class WithSQLiteTest(unittest.TestCase):
    def setUp(self):
        self.db = sqlite_db('test')
        self.db.begin()

    def tearDown(self):
        self.db.clear()
        self.db.save()

    def test_empty(self):
        self.assertEqual(self.db.items(), [])

    def test_add_one_string(self):
        self.db['a'] = 'test'
        items = self.db.items()
        self.assertEqual(len(items), 1)
        k, v = items[0]
        self.assertEqual(k, 'a')
        self.assertEqual(v, 'test')

    def test_add_two_strings(self):
        self.db['a'] = 'test_a'
        self.db['b'] = 'test_b'
        items = self.db.items()
        self.assertEqual(len(items), 2)
        k, v = items[0]
        self.assertEqual(k, 'a')
        self.assertEqual(v, 'test_a')
        k, v = items[1]
        self.assertEqual(k, 'b')
        self.assertEqual(v, 'test_b')

    def test_add_a_list(self):
        self.db['list'] = [1, 2, 3, 4, 5]
        items = self.db.items()
        self.assertEqual(len(items), 1)
        k, v = items[0]
        self.assertEqual(k, 'list')
        self.assertEqual(v, [1, 2, 3, 4, 5])

    def test_del_value(self):
        self.db['a'] = 'value_a'
        self.db['some'] = 'to be deleted'
        self.assertEqual(len(self.db.items()), 2)
        del self.db['some']
        self.assertEqual(len(self.db.items()), 1)
        self.assertIsNone(self.db.get('some'))
        try:
            self.db['some']
            self.assertEqual(1, 0, "__getitem__")
        except KeyError:
            pass

    def test_keys(self):
        values = ['a', 'b', 'c']
        for val in values:
            self.db[val] = val.upper()
        self.assertEqual(set(self.db.keys()), set(values))

    def test_values(self):
        values = ['A', 'B', 'C']
        for val in values:
            self.db[val.lower()] = val
        self.assertEqual(set(self.db.values()), set(values))

    def test_get(self):
        self.assertEqual(self.db.get('some', 5), 5)

    def test_repr(self):
        adict = {'a': 'A'}
        self.db['a'] = 'A'
        self.assertEqual(repr(adict), repr(self.db))

    def test_contains(self):
        self.db['key'] = 'value'
        self.assertTrue('key' in self.db)
        self.assertFalse('value' in self.db)

    def test_update(self):
        self.db['key'] = 'value'
        self.assertEqual(self.db['key'], 'value')
        self.db['key'] = 'foobar'
        self.assertEqual(self.db['key'], 'foobar')

    def test_dir(self):
        workdir = tempfile.mkdtemp()
        with sqlite_db("test", dir=workdir) as db:
            db['testing'] = 'dir'
        with sqlite_db("test", dir=workdir) as db:
            self.assertEqual(db['testing'], 'dir')

    def test_transaction(self):
        a = sqlite_db("test")
        with a.transaction():
            a['key'] = 'value'
            self.assertEqual(len(a.items()), 1)
            self.assertEqual(len(self.db.items()), 0)
        items = self.db.items()
        self.assertEqual(len(items), 1)
        k, v = items[0]
        self.assertEqual('key', k)
        self.assertEqual('value', v)

    def test_with(self):
        with sqlite_db("test") as db:
            db['a'] = 'test_a'
            db['b'] = 'test_b'
            items = db.items()
            self.assertEqual(len(items), 2)
            k, v = items[0]
            self.assertEqual(k, 'a')
            self.assertEqual(v, 'test_a')
            k, v = items[1]
            self.assertEqual(k, 'b')
            self.assertEqual(v, 'test_b')

    def test_multitable(self):
        store_file = "test"
        with sqlite_db(store_file, table="firstboot", autocommit=True) as db:
            db['state']=1
        with sqlite_db(store_file, table="thisbox", autocommit=True) as db:
            db['state']=1
